from fastapi import FastAPI, Body

from spacy import displacy
from .models import get_model
from .clean_text import text_cleaner, text_formatter


app = FastAPI(
    title="Name extractor using Spacy",
    description="Extract all names from a row text",
    version="0.1",
)
model = get_model()




@app.get("/")
async def home():
    return {"status": "Server is Up!"}


@app.post('/name')
def is_a_human_name(data: dict = Body(...)):
    text = data.get('name')
    doc = model(text)
    is_name = False
    for ent in doc.ents:
        if ent.label_ in ['PERSON']:
            is_name = True

    response =  {
        'name': is_name
    }
    return response


@app.post('/names/all')
def get_all_names(data: dict = Body(...)):
    text = data.get('text')
    text = text_cleaner(text)
    text = text_formatter(text)
    doc = model(text)
    names = {
        ent.text for ent in doc.ents if ent.label_ in ['PERSON', 'ORG', 'PRODUCT']
    }

    response =  {
        'names': names
    }
    return response



@app.post('/displacy/google')
def get_html_entity(data: dict = Body(...)):

    reviews = data.get('reviews')

    for rev in reviews:
        doc = model(text_formatter(rev['comment']))
        options = {"ents": ["ORG", "PERSON", "GPE", "PRODUCT", "DATE", "TIME", "MONEY"]}
        html = displacy.render(doc, options=options, style='ent')
        rev['comment'] = html
    return reviews